@@ -6,12 +6,14 @@ from trytond.model import ModelView, ModelSQL, fields
 from trytond.tools import safe_eval, datetime_strftime
 from trytond.transaction import Transaction
 from trytond.pool import Pool
+from trytond.rpc import RPC
+
+__all__ = ['Currency', 'Rate']
 
 
 class Currency(ModelSQL, ModelView):
     'Currency'
-    _name = 'currency.currency'
-    _description = __doc__
+    __name__ = 'currency.currency'
     name = fields.Char('Name', required=True, translate=True)
     symbol = fields.Char('Symbol', size=10, required=True)
     code = fields.Char('Code', size=3, required=True)
@@ -36,95 +38,112 @@ class Currency(ModelSQL, ModelView):
     p_sep_by_space = fields.Boolean('Positive Separate by Space')
     n_sep_by_space = fields.Boolean('Negative Separate by Space')
 
-    def __init__(self):
-        super(Currency, self).__init__()
-        self._order.insert(0, ('code', 'ASC'))
-        self._constraints += [
+    @classmethod
+    def __setup__(cls):
+        super(Currency, cls).__setup__()
+        cls._order.insert(0, ('code', 'ASC'))
+        cls._constraints += [
             ('check_mon_grouping', 'invalid_mon_grouping'),
-        ]
-        self._error_messages.update({
-            'no_rate': 'No rate found for the currency: %s at the date: %s',
-            'invalid_grouping': 'Invalid Grouping!',
-            })
-        self._rpc.update({
-            'compute': False,
-            })
+            ]
+        cls._error_messages.update({
+                'no_rate': 'No rate found for the currency: ' \
+                    '%s at the date: %s',
+                'invalid_grouping': 'Invalid Grouping!',
+                })
+        cls.__rpc__.update({
+                'compute': RPC(),
+                })
 
-    def default_active(self):
+    @staticmethod
+    def default_active():
         return True
 
-    def default_rounding(self):
+    @staticmethod
+    def default_rounding():
         return Decimal('0.01')
 
-    def default_digits(self):
+    @staticmethod
+    def default_digits():
         return 2
 
-    def default_mon_grouping(self):
+    @staticmethod
+    def default_mon_grouping():
         return '[]'
 
-    def default_mon_thousands_sep(self):
+    @staticmethod
+    def default_mon_thousands_sep():
         return ','
 
-    def default_mon_decimal_point(self):
+    @staticmethod
+    def default_mon_decimal_point():
         return '.'
 
-    def default_p_sign_posn(self):
+    @staticmethod
+    def default_p_sign_posn():
         return 1
 
-    def default_n_sign_posn(self):
+    @staticmethod
+    def default_n_sign_posn():
         return 1
 
-    def default_negative_sign(self):
+    @staticmethod
+    def default_negative_sign():
         return '-'
 
-    def default_positive_sign(self):
+    @staticmethod
+    def default_positive_sign():
         return ''
 
-    def default_p_cs_precedes(self):
+    @staticmethod
+    def default_p_cs_precedes():
         return True
 
-    def default_n_cs_precedes(self):
+    @staticmethod
+    def default_n_cs_precedes():
         return True
 
-    def default_p_sep_by_space(self):
+    @staticmethod
+    def default_p_sep_by_space():
         return False
 
-    def default_n_sep_by_space(self):
+    @staticmethod
+    def default_n_sep_by_space():
         return False
 
-    def check_mon_grouping(self, ids):
+    def check_mon_grouping(self):
         '''
         Check if mon_grouping is list of numbers
         '''
-        for currency in self.browse(ids):
-            try:
-                grouping = safe_eval(currency.mon_grouping)
-                for i in grouping:
-                    if not isinstance(i, int):
-                        return False
-            except Exception:
-                return False
+        try:
+            grouping = safe_eval(self.mon_grouping)
+            for i in grouping:
+                if not isinstance(i, int):
+                    return False
+        except Exception:
+            return False
         return True
 
-    def check_xml_record(self, ids, values):
+    @classmethod
+    def check_xml_record(cls, records, values):
         return True
 
-    def search_rec_name(self, name, clause):
-        ids = []
+    @classmethod
+    def search_rec_name(cls, name, clause):
+        currencies = None
         field = None
         for field in ('code', 'numeric_code'):
-            ids = self.search([(field,) + clause[1:]], limit=1)
-            if ids:
+            currencies = cls.search([(field,) + clause[1:]], limit=1)
+            if currencies:
                 break
-        if ids:
+        if currencies:
             return [(field,) + clause[1:]]
-        return [(self._rec_name,) + clause[1:]]
+        return [(cls._rec_name,) + clause[1:]]
 
-    def on_change_with_rate(self, vals):
+    def on_change_with_rate(self):
         now = datetime.date.today()
         closer = datetime.date.min
         res = Decimal('0.0')
-        for rate in vals.get('rates', []):
+        for rate in self.rates or []:
             if 'date' not in rate or 'rate' not in rate:
                 continue
             if rate['date'] <= now and rate['date'] > closer:
@@ -132,26 +151,27 @@ class Currency(ModelSQL, ModelView):
                 closer = rate['date']
         return res
 
-    def get_rate(self, ids, name):
+    @staticmethod
+    def get_rate(currencies, name):
         '''
         Return the rate at the date from the context or the current date
         '''
-        rate_obj = Pool().get('currency.currency.rate')
-        date_obj = Pool().get('ir.date')
+        Rate = Pool().get('currency.currency.rate')
+        Date = Pool().get('ir.date')
 
         res = {}
-        date = Transaction().context.get('date', date_obj.today())
-        for currency_id in ids:
-            rate_ids = rate_obj.search([
-                ('currency', '=', currency_id),
-                ('date', '<=', date),
-                ], limit=1, order=[('date', 'DESC')])
-            if rate_ids:
-                res[currency_id] = rate_ids[0]
+        date = Transaction().context.get('date', Date.today())
+        for currency in currencies:
+            rates = Rate.search([
+                    ('currency', '=', currency.id),
+                    ('date', '<=', date),
+                    ], limit=1, order=[('date', 'DESC')])
+            if rates:
+                res[currency.id] = rates[0].id
             else:
-                res[currency_id] = 0.0
+                res[currency.id] = 0.0
         rate_ids = [x for x in res.values() if x]
-        rates = rate_obj.browse(rate_ids)
+        rates = Rate.browse(rate_ids)
         id2rate = {}
         for rate in rates:
             id2rate[rate.id] = rate
@@ -160,7 +180,7 @@ class Currency(ModelSQL, ModelView):
                 res[currency_id] = id2rate[res[currency_id]].rate
         return res
 
-    def round(self, currency, amount, rounding=ROUND_HALF_EVEN):
+    def round(self, amount, rounding=ROUND_HALF_EVEN):
         '''
         Round the amount depending of the currency
 
@@ -171,80 +191,76 @@ class Currency(ModelSQL, ModelView):
         :param rounding: the rounding option
         :return: a Decimal
         '''
-        return (amount / currency.rounding).quantize(Decimal('1.'),
-                rounding=rounding) * currency.rounding
+        return (amount / self.rounding).quantize(Decimal('1.'),
+                rounding=rounding) * self.rounding
 
-    def is_zero(self, currency, amount):
+    def is_zero(self, amount):
         'Return True if the amount can be considered as zero for the currency'
-        return abs(self.round(currency, amount)) < currency.rounding
+        return abs(self.round(amount)) < self.rounding
 
-    def compute(self, from_currency, amount, to_currency, round=True):
+    @classmethod
+    def compute(cls, from_currency, amount, to_currency, round=True):
         '''
         Take a currency and an amount
         Return the amount to the new currency
         Use the rate of the date of the context or the current date if ids are
         given
         '''
-        date_obj = Pool().get('ir.date')
-        lang_obj = Pool().get('ir.lang')
+        Date = Pool().get('ir.date')
+        Lang = Pool().get('ir.lang')
+        from_currency = cls(from_currency.id)
+        to_currency = cls(to_currency.id)
 
-        if isinstance(from_currency, (int, long)):
-            from_currency = self.browse(from_currency)
-        if isinstance(to_currency, (int, long)):
-            to_currency = self.browse(to_currency)
         if to_currency == from_currency:
             if round:
-                return self.round(to_currency, amount)
+                return to_currency.round(amount)
             else:
                 return amount
         if (not from_currency.rate) or (not to_currency.rate):
-            date = Transaction().context.get('date', date_obj.today())
+            date = Transaction().context.get('date', Date.today())
             if not from_currency.rate:
                 name = from_currency.name
             else:
                 name = to_currency.name
 
-            lang_id, = lang_obj.search([
+            lang, = Lang.search([
                     ('code', '=', Transaction().language),
                     ])
-            lang = lang_obj.browse(lang_id)
 
-            self.raise_user_error('no_rate', (name,
-                datetime_strftime(date, str(lang.date))))
+            cls.raise_user_error('no_rate', (name,
+                    datetime_strftime(date, str(lang.date))))
         if round:
-            return self.round(to_currency,
-                    amount * to_currency.rate / from_currency.rate)
+            return to_currency.round(
+                amount * to_currency.rate / from_currency.rate)
         else:
             return amount * to_currency.rate / from_currency.rate
 
-Currency()
-
 
 class Rate(ModelSQL, ModelView):
     'Rate'
-    _name = 'currency.currency.rate'
-    _description = __doc__
+    __name__ = 'currency.currency.rate'
     _rec_name = 'date'
     date = fields.Date('Date', required=True, select=True)
     rate = fields.Numeric('Rate', digits=(12, 6), required=1)
     currency = fields.Many2One('currency.currency', 'Currency',
             ondelete='CASCADE',)
 
-    def __init__(self):
-        super(Rate, self).__init__()
-        self._sql_constraints = [
+    @classmethod
+    def __setup__(cls):
+        super(Rate, cls).__setup__()
+        cls._sql_constraints = [
             ('date_currency_uniq', 'UNIQUE(date, currency)',
                 'A currency can only have one rate by date!'),
             ('check_currency_rate', 'CHECK(rate >= 0)',
                 'The currency rate must greater than or equal to 0'),
-        ]
-        self._order.insert(0, ('date', 'DESC'))
+            ]
+        cls._order.insert(0, ('date', 'DESC'))
 
-    def default_date(self):
-        date_obj = Pool().get('ir.date')
-        return date_obj.today()
+    @staticmethod
+    def default_date():
+        Date = Pool().get('ir.date')
+        return Date.today()
 
-    def check_xml_record(self, ids, values):
+    @classmethod
+    def check_xml_record(self, records, values):
         return True
-
-Rate()
